<?php
/*
NOTICE OF LICENSE

This source file is subject to the NekloEULA that is bundled with this package in the file ICENSE.txt.

It is also available through the world-wide-web at this URL: http://store.neklo.com/LICENSE.txt

Copyright (c)  Neklo (http://store.neklo.com/)
*/


namespace Neklo\Scheduler\Controller\Adminhtml\Schedule;

use Neklo\Scheduler\Controller\Adminhtml;
use Magento\Backend\App\Action\Context;

class MassDelete extends Adminhtml\Action
{
    const ADMIN_RESOURCE = 'Neklo_Scheduler::edit';
    /**
     * Redirect url
     */
    const REDIRECT_URL = '*/schedule/index';

    /**
     * Field id
     */
    const ID_FIELD = 'selected';

    /**
     *
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     *
     * @var \Neklo\Scheduler\Model\JobFactory
     */
    private $jobFactory;

    public function __construct(
        Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Registry $registry,
        \Magento\Cron\Model\ScheduleFactory $modelFactory,
        \Psr\Log\LoggerInterface $logger
    ) {
        $this->logger = $logger;
        $this->jobFactory = $modelFactory;
        parent::__construct($context, $resultPageFactory, $registry);
    }

    public function execute()
    {
        $selected = $this->getSelected();
        try {
            if (!empty($selected)) {
                $this->setArray($selected);
            } else {
                $this->messageManager->addError(__('Please select item(s).'));
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            $this->messageManager->addError(__('Operation error'));
        }

        return $this->_redirect(static::REDIRECT_URL);
    }

    private function setArray($items)
    {
        $errMsg = [];
        foreach ($items as $id) {
            $job = $this->jobFactory->create();
            try {
                $job->load($id);
                if ($job->getId()) {
                    $job->delete($job);
                }
            } catch (\Exception $e) {
                $err = true;
                $this->logger->critical($e->getMessage());
                $errMsg[] = __('Job ID: %1 does not exist', $id);
            }
        }

        if (empty($err)) {
            $this->messageManager->addSuccess(__('A Total of %1 Record(s) have been Deleted', count($items)));
        } else {
            $this->messageManager->addError(implode('<br>', $errMsg));
        }
    }

    protected function getSelected()
    {
        $selected = $this->getRequest()->getParam(self::ID_FIELD);
        $excluded = $this->getRequest()->getParam('excluded');
        if ($excluded !== null) {
            $selected = $this->jobFactory->create()
                ->getCollection()
                ->addFieldToFilter('main_table.schedule_id', ['nin' => $excluded])
                ->getAllIds();
        }
        if (!is_array($selected)) {
            $selected = explode(',', $selected);
        }

        return $selected;
    }
}
