<?php
/*
NOTICE OF LICENSE

This source file is subject to the NekloEULA that is bundled with this package in the file ICENSE.txt.

It is also available through the world-wide-web at this URL: http://store.neklo.com/LICENSE.txt

Copyright (c)  Neklo (http://store.neklo.com/)
*/


namespace Neklo\Scheduler\Observer;

use Exception;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Event\Observer;
use Neklo\Scheduler\Api\Data\JobInterface;
use Neklo\Scheduler\Model\Job;
use Neklo\Scheduler\Model\JobFactory;
use Psr\Log\LoggerInterface;

/**
 * Class ProcessCronQueueObserver
 * @package Neklo\Scheduler\Observer
 */
class ProcessCronQueueObserver extends \Magento\Cron\Observer\ProcessCronQueueObserver
{
    /**
     * Process cron queue
     * Generate tasks schedule
     * Cleanup tasks schedule
     *
     * @param Observer $observer
     * @return void
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function execute(Observer $observer)
    {
        parent::execute($observer);
        $this->updateJobs();
    }

    /**
     *
     */
    private function updateJobs()
    {
        $objectManager = ObjectManager::getInstance();
        /** @var LoggerInterface $logger */
        $logger = $objectManager->create(LoggerInterface::class);

        /** @var JobFactory $jobGroups */
        $jobGroups = $this->_config->getJobs();

        foreach ($jobGroups as $group => $jobs) {
            foreach ($jobs as $jobName => $job) {
                try {
                    /** @var Job $model */
                    $model = $objectManager->create(Job::class);
                    $model = $model->load($jobName, JobInterface::JOB_NAME);
                    if (!$model->getId()) {
                        $model->setName($jobName)
                            ->setMethod($job[JobInterface::JOB_METHOD])
                            ->setInstance($job[JobInterface::JOB_CLASS])
                            ->setEnabled(false)
                            ->setGroup($group);
                        if (isset($job[JobInterface::JOB_SCHEDULE])) {
                            $model->setSchedule($job[JobInterface::JOB_SCHEDULE]);
                        }

                        if (isset($job[JobInterface::JOB_CONFIG])) {
                            $model->setConfigPath($job[JobInterface::JOB_CONFIG]);
                        }

                        $model->save();
                    }
                } catch (Exception $e) {
                    $logger->error($e->getMessage(), ['exception' => $e]);
                }
            }
        }
    }

    /**
     * @param array $jobs
     * @param array $exists
     * @param string $groupId
     */
    protected function _generateJobs($jobs, $exists, $groupId)
    {
        $jobsToSchedule = [];

        foreach ($jobs as $jobCode => $jobConfig) {

            if (!isset($jobConfig[JobInterface::JOB_ENABLED])) {
                continue;
            }

            if (false === (bool)$jobConfig[JobInterface::JOB_ENABLED]) {
                continue;
            }

            $jobsToSchedule[$jobCode] = $jobConfig;
        }

        parent::_generateJobs($jobsToSchedule, $exists, $groupId);
    }
}
